@@ -2,7 +2,7 @@ use strict;
 use warnings;
 package LaTeX::ToUnicode;
 BEGIN {
-  $LaTeX::ToUnicode::VERSION = '0.53';
+  $LaTeX::ToUnicode::VERSION = '1.92';
 }
 #ABSTRACT: Convert LaTeX commands to Unicode (simplistically)
 
@@ -19,14 +19,19 @@ use LaTeX::ToUnicode::Tables;
 # and then ignore any following whitespace.
 our $endcw = qr/(?<=[a-zA-Z])(?=[^a-zA-Z]|$)\s*/;
 
-# all we need for is debugging being on and off. And it's pretty random
-# what gets output.
+# Debugging output on or off; it's pretty random what gets output.
+# Add more as needed. There is also more debugging output as warn
+# statements, commented out, too voluminous to enable here.
 my $debug = 0;
+
 sub debuglevel { $debug = shift; }
 sub _debug {
   return unless $debug;
-  my ($pkgname,$filename,$line,$subr) = caller(1);
-  warn @_, " at $filename:$line (${pkgname}::$subr)\n";
+  # The backtrace info is split between caller(0) and caller(1), sigh.
+  # We don't need the package name, it's included in $subr in practice.
+  my (undef,$filename,$line,undef) = caller(0);
+  my (undef,undef,undef,$subr) = caller(1);
+  warn @_, " at $filename:$line ($subr)\n";
 }
 
 # The main conversion function.
@@ -55,6 +60,7 @@ sub convert {
     
     my $user_hook = $options{hook};
     if ($user_hook) {
+        _debug("before user hook: $string");
         $string = &$user_hook($string, \%options);
         _debug("after user hook: $string");
     }
@@ -88,24 +94,35 @@ sub convert {
     $string = _convert_ligatures($string);
     
     # Let's handle ties here, after all the other conversions, since
-    # they don't fit well with any of the tables.
+    # they don't fit well with any of the tables. We don't handle TeX's
+    # other special characters: $ & # ^ _.
     # 
     # /~, or ~ at the beginning of a line, is probably part of a url or
-    # path, not a tie. Otherwise, consider it a space, since a no-break
-    # spot in TeX is most likely fine to break in text or HTML.
+    # path, not a tie. Otherwise, consider it a space, since we can't
+    # distinguish true no-break spots (Donald~E. Knuth) from ties that
+    # are only relevant to a particular line width.
     # 
     $string =~ s,([^/])~,$1 ,g;
     
     # Remove kerns. Clearly needs generalizing/sharpening to recognize
     # dimens better, and plenty of other commands could use it.
+    # Here, we only handle literal dimensions ("+1.3pt"), not dimens
+    # referring to control sequences, with or without factors
+    # ("1.1\baselineskip").
     #_debug("before kern: $string");
     my $dimen_re = qr/[-+]?[0-9., ]+[a-z][a-z]\s*/;
     $string =~ s!\\kern${endcw}${dimen_re}!!g;
     
     # What the heck, let's do \hfuzz and \vfuzz too. They come up pretty
-    # often and practically the same thing (plus ignore optional =)..
+    # often and it's practically the same thing (just also ignore optional =)..
     $string =~ s!\\[hv]fuzz${endcw}=?\s*${dimen_re}!!g;    
 
+    # And here is \penalty. natbib outputs \penalty0 sometimes.
+    # Similar with $dimen_re, we only handle literal and decimal
+    # integers here, not things like "0 or `A.
+    my $number_re = qr/[-+]?[0-9]+\s*/;
+    $string =~ s!\\penalty${endcw}\s*${number_re}!!g;    
+
     # After all the conversions, $string contains \x{....} constructs
     # (Perl Unicode characters) where translations have happened. Change
     # those to the desired output format. Thus we assume that the
@@ -114,8 +131,17 @@ sub convert {
     # 
     if (! $options{entities}) {
       # Convert our \x strings from Tables.pm to the binary characters.
-      # Assume no more than four hex digits.
-      $string =~ s/\\x\{(.{1,4})\}/ pack('U*', hex($1))/eg;
+      
+      # As an extra-special case, we want to preserve the translation of
+      # \{ and \} as 007[bd] entities even if the --entities option is
+      # not give; otherwise they'd get eliminated like all other braces.
+      # Use a temporary cs \xx to keep them marked, and don't use braces
+      # to delimit the argument since they'll get deleted.
+      $string =~ s/\\x\{(007[bd])\}/\\xx($1)/g;
+      
+      # Convert all other characters to characters.
+      # Assume exactly four hex digits, since we wrote Tables.pm that way.
+      $string =~ s/\\x\{(....)\}/ pack('U*', hex($1))/eg;
 
     } elsif ($options{entities}) {
       # Convert the XML special characters that appeared in the input,
@@ -176,9 +202,15 @@ sub convert {
       warn "LaTeX::ToUnicode::convert:   please report as bug.\n";
     }
     
-    # Drop all braces.
+    # Drop all remaining braces.
     $string =~ s/[{}]//g;
     
+    if (! $options{entities}) {
+      # With all the other braces gone, now we can convert the preserved
+      # brace entities from \{ and \} to actual braces.
+      $string =~ s/\\xx\((007[bd])\)/ pack('U*', hex($1))/eg;
+    }
+
     # Backslashes might remain. Don't remove them, as it makes for a
     # useful way to find unhandled commands.
 
@@ -247,16 +279,16 @@ sub _convert_urls {
         # removed.  We want to accept and ignore such extra braces,
         # hence the \{+ ... \}+ in recognizing TEXT.
         # 
-#warn "txt url: starting with $string\n";
+        #warn "txt url: starting with $string\n";
         if ($string =~ m/\\href$endcw\{([^}]*)\}\s*\{+([^}]*)\}+/) {
           my $url = $1;
           my $text = $2;
-#warn "   url: $url\n";
-#warn "  text: $text\n";
+          #warn "    url: $url\n";
+          #warn "   text: $text\n";
           my $repl = ($url =~ m!$text$!) ? $url : "$text ($url)";
-#warn "  repl: $repl\n";
+          #warn "   repl: $repl\n";
           $string =~ s/\\href$endcw\{([^}]*)\}\s*\{+([^}]*)\}+/$repl/;
-#warn "  str:  $string\n";
+          #warn " result:  $string\n";
         }
     }
     
@@ -325,7 +357,7 @@ sub _convert_accents {
     $string =~ s/(\\(.)\s*(\\?\w{1,1}))/        $tbl{$2}{$3} || $1/eg; # \"a
     $string =~ s/(\\(.)\s*\{(\\?\w{1,2})\})/    $tbl{$2}{$3} || $1/eg; # \"{a}
     
-    # second the alphabetic commands, like \c. They have be handled
+    # second the alphabetic commands, like \c. They have to be handled
     # differently because \cc is not \c{c}! The only difference in the
     # regular expressions is using $endcw instead of just \s*.
     # 
@@ -413,22 +445,31 @@ sub _convert_markups {
     
     # HTML is different.
     return _convert_markups_html($string) if $options->{html};
-    # Ok, we'll "convert" to plain text by removing the markup commands.
+    
+    # Not HTML, so here we'll "convert" to plain text by removing the
+    # markup commands.
 
-    # we can do all markup commands at once.
+    # we can do all the markup commands at once.
     my $markups = join('|', keys %LaTeX::ToUnicode::Tables::MARKUPS);
     
+    #warn "_convert_markups: markups = $markups\n";
+    #warn "_convert_markups plain text: starting with $string\n";
     # Remove \textMARKUP{...}, leaving just the {...}
     $string =~ s/\\text($markups)$endcw//g;
+    #warn " after \text: $string\n";
 
-    # Similarly remove \MARKUPshape. 
-    $string =~ s/\\($markups)shape$endcw//g;
+    # Similarly remove \MARKUPshape, plus remove \upshape.
+    $string =~ s/\\($markups|up)shape$endcw//g;
+    #warn " after \...shape: $string\n";
 
     # Remove braces and \command in: {... \MARKUP ...}
+    # where neither ... can contain braces.
     $string =~ s/(\{[^{}]+)\\(?:$markups)$endcw([^{}]+\})/$1$2/g;
+    #warn " after ...\\markup...: $string\n";
 
     # Remove braces and \command in: {\MARKUP ...}
     $string =~ s/\{\\(?:$markups)$endcw([^{}]*)\}/$1/g;
+    #warn " after {\\markup...}: $string\n";
 
     # Remove: {\MARKUP
     # Although this will leave unmatched } chars behind, there's no
@@ -436,9 +477,10 @@ sub _convert_markups {
     # look like: {\em {The TeX{}book}}. Also might, in principle, be
     # at the end of a line.
     $string =~ s/\{\\(?:$markups)$endcw//g;
+    #warn " after {\\markup: $string\n";
 
     # Ultimately we remove all braces in ltx2crossrefxml SanitizeText fns,
-    # so the unmatched braces don't matter ... that code should be moved.
+    # so the unmatched braces don't matter ... that code should be moved here.
 
     $string;
 }
@@ -516,7 +558,7 @@ LaTeX::ToUnicode - Convert LaTeX commands to Unicode
 
 =head1 VERSION
 
-version 0.53
+version 1.92
 
 =head1 SYNOPSIS
 
@@ -749,7 +791,7 @@ L<https://github.com/borisveytsman/bibtexperllibs>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2010-2023 Gerhard Gossen, Boris Veytsman, Karl Berry
+Copyright 2010-2025 Gerhard Gossen, Boris Veytsman, Karl Berry
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl5 programming language system itself.
